@@ -20,7 +20,7 @@ package org.apache.kyuubi.plugin.spark.authz.ranger
 import scala.util.Try
 
 import org.apache.hadoop.security.UserGroupInformation
-import org.apache.spark.sql.{Row, SparkSessionExtensions}
+import org.apache.spark.sql.SparkSessionExtensions
 import org.apache.spark.sql.catalyst.analysis.NoSuchTableException
 import org.apache.spark.sql.catalyst.catalog.HiveTableRelation
 import org.apache.spark.sql.catalyst.plans.logical.Statistics
@@ -31,9 +31,11 @@ import org.scalatest.BeforeAndAfterAll
 import org.scalatest.funsuite.AnyFunSuite
 
 import org.apache.kyuubi.plugin.spark.authz.{AccessControlException, SparkSessionProvider}
+import org.apache.kyuubi.plugin.spark.authz.RangerTestNamespace._
+import org.apache.kyuubi.plugin.spark.authz.RangerTestUsers._
 import org.apache.kyuubi.plugin.spark.authz.ranger.RuleAuthorization.KYUUBI_AUTHZ_TAG
-import org.apache.kyuubi.plugin.spark.authz.util.AuthZUtils.getFieldVal
-
+import org.apache.kyuubi.plugin.spark.authz.util.AuthZUtils._
+import org.apache.kyuubi.util.reflect.ReflectUtils._
 abstract class RangerSparkExtensionSuite extends AnyFunSuite
   with SparkSessionProvider with BeforeAndAfterAll {
   // scalastyle:on
@@ -87,8 +89,23 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     }
   }
 
+  /**
+   * Enables authorizing in single call mode,
+   * and disables authorizing in single call mode after calling `f`
+   */
+  protected def withSingleCallEnabled(f: => Unit): Unit = {
+    val singleCallConfig =
+      s"ranger.plugin.${SparkRangerAdminPlugin.getServiceType}.authorize.in.single.call"
+    try {
+      SparkRangerAdminPlugin.getRangerConf.setBoolean(singleCallConfig, true)
+      f
+    } finally {
+      SparkRangerAdminPlugin.getRangerConf.setBoolean(singleCallConfig, false)
+    }
+  }
+
   test("[KYUUBI #3226] RuleAuthorization: Should check privileges once only.") {
-    val logicalPlan = doAs("admin", sql("SHOW TABLES").queryExecution.logical)
+    val logicalPlan = doAs(admin, sql("SHOW TABLES").queryExecution.logical)
     val rule = new RuleAuthorization(spark)
 
     (1 until 10).foreach { i =>
@@ -116,7 +133,7 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     withCleanTmpResources(Seq((testTable, "table"))) {
       // create tmp table
       doAs(
-        "admin", {
+        admin, {
           sql(create)
 
           // session1: first query, should auth once.[LogicalRelation]
@@ -155,18 +172,18 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     val e = intercept[AccessControlException](sql(create))
     assert(e.getMessage === errorMessage("create", "mydb"))
     withCleanTmpResources(Seq((testDb, "database"))) {
-      doAs("admin", assert(Try { sql(create) }.isSuccess))
-      doAs("admin", assert(Try { sql(alter) }.isSuccess))
+      doAs(admin, assert(Try { sql(create) }.isSuccess))
+      doAs(admin, assert(Try { sql(alter) }.isSuccess))
       val e1 = intercept[AccessControlException](sql(alter))
       assert(e1.getMessage === errorMessage("alter", "mydb"))
       val e2 = intercept[AccessControlException](sql(drop))
       assert(e2.getMessage === errorMessage("drop", "mydb"))
-      doAs("kent", Try(sql("SHOW DATABASES")).isSuccess)
+      doAs(kent, Try(sql("SHOW DATABASES")).isSuccess)
     }
   }
 
   test("auth: tables") {
-    val db = "default"
+    val db = defaultDb
     val table = "src"
     val col = "key"
 
@@ -178,14 +195,14 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     assert(e.getMessage === errorMessage("create"))
 
     withCleanTmpResources(Seq((s"$db.$table", "table"))) {
-      doAs("bob", assert(Try { sql(create0) }.isSuccess))
-      doAs("bob", assert(Try { sql(alter0) }.isSuccess))
+      doAs(bob, assert(Try { sql(create0) }.isSuccess))
+      doAs(bob, assert(Try { sql(alter0) }.isSuccess))
 
       val e1 = intercept[AccessControlException](sql(drop0))
       assert(e1.getMessage === errorMessage("drop"))
-      doAs("bob", assert(Try { sql(alter0) }.isSuccess))
-      doAs("bob", assert(Try { sql(select).collect() }.isSuccess))
-      doAs("kent", assert(Try { sql(s"SELECT key FROM $db.$table").collect() }.isSuccess))
+      doAs(bob, assert(Try { sql(alter0) }.isSuccess))
+      doAs(bob, assert(Try { sql(select).collect() }.isSuccess))
+      doAs(kent, assert(Try { sql(s"SELECT key FROM $db.$table").collect() }.isSuccess))
 
       Seq(
         select,
@@ -196,10 +213,10 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
         s"SELECT key FROM $db.$table WHERE value in (SELECT value as key FROM $db.$table)")
         .foreach { q =>
           doAs(
-            "kent", {
+            kent, {
               withClue(q) {
                 val e = intercept[AccessControlException](sql(q).collect())
-                assert(e.getMessage === errorMessage("select", "default/src/value", "kent"))
+                assert(e.getMessage === errorMessage("select", "default/src/value", kent))
               }
             })
         }
@@ -207,92 +224,15 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
   }
 
   test("auth: functions") {
-    val db = "default"
+    val db = defaultDb
     val func = "func"
     val create0 = s"CREATE FUNCTION IF NOT EXISTS $db.$func AS 'abc.mnl.xyz'"
     doAs(
-      "kent", {
+      kent, {
         val e = intercept[AccessControlException](sql(create0))
         assert(e.getMessage === errorMessage("create", "default/func"))
       })
-    doAs("admin", assert(Try(sql(create0)).isSuccess))
-  }
-
-  test("row level filter") {
-    val db = "default"
-    val table = "src"
-    val col = "key"
-    val create = s"CREATE TABLE IF NOT EXISTS $db.$table ($col int, value int) USING $format"
-
-    withCleanTmpResources(Seq((s"$db.${table}2", "table"), (s"$db.$table", "table"))) {
-      doAs("admin", assert(Try { sql(create) }.isSuccess))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 1, 1"))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 20, 2"))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 30, 3"))
-
-      doAs(
-        "kent",
-        assert(sql(s"SELECT key FROM $db.$table order by key").collect() ===
-          Seq(Row(1), Row(20), Row(30))))
-
-      Seq(
-        s"SELECT value FROM $db.$table",
-        s"SELECT value as key FROM $db.$table",
-        s"SELECT max(value) FROM $db.$table",
-        s"SELECT coalesce(max(value), 1) FROM $db.$table",
-        s"SELECT value FROM $db.$table WHERE value in (SELECT value as key FROM $db.$table)")
-        .foreach { q =>
-          doAs(
-            "bob", {
-              withClue(q) {
-                assert(sql(q).collect() === Seq(Row(1)))
-              }
-            })
-        }
-      doAs(
-        "bob", {
-          sql(s"CREATE TABLE $db.src2 using $format AS SELECT value FROM $db.$table")
-          assert(sql(s"SELECT value FROM $db.${table}2").collect() === Seq(Row(1)))
-        })
-    }
-  }
-
-  test("[KYUUBI #3581]: row level filter on permanent view") {
-    assume(isSparkV31OrGreater)
-
-    val db = "default"
-    val table = "src"
-    val permView = "perm_view"
-    val col = "key"
-    val create = s"CREATE TABLE IF NOT EXISTS $db.$table ($col int, value int) USING $format"
-    val createView =
-      s"CREATE OR REPLACE VIEW $db.$permView" +
-        s" AS SELECT * FROM $db.$table"
-
-    withCleanTmpResources(Seq(
-      (s"$db.$table", "table"),
-      (s"$db.$permView", "view"))) {
-      doAs("admin", assert(Try { sql(create) }.isSuccess))
-      doAs("admin", assert(Try { sql(createView) }.isSuccess))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 1, 1"))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 20, 2"))
-      doAs("admin", sql(s"INSERT INTO $db.$table SELECT 30, 3"))
-
-      Seq(
-        s"SELECT value FROM $db.$permView",
-        s"SELECT value as key FROM $db.$permView",
-        s"SELECT max(value) FROM $db.$permView",
-        s"SELECT coalesce(max(value), 1) FROM $db.$permView",
-        s"SELECT value FROM $db.$permView WHERE value in (SELECT value as key FROM $db.$permView)")
-        .foreach { q =>
-          doAs(
-            "perm_view_user", {
-              withClue(q) {
-                assert(sql(q).collect() === Seq(Row(1)))
-              }
-            })
-        }
-    }
+    doAs(admin, assert(Try(sql(create0)).isSuccess))
   }
 
   test("show tables") {
@@ -303,14 +243,14 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
         (s"$db.$table", "table"),
         (s"$db.${table}for_show", "table"),
         (s"$db", "database"))) {
-      doAs("admin", sql(s"CREATE DATABASE IF NOT EXISTS $db"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.$table (key int) USING $format"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}for_show (key int) USING $format"))
+      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $db"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.$table (key int) USING $format"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}for_show (key int) USING $format"))
 
-      doAs("admin", assert(sql(s"show tables from $db").collect().length === 2))
-      doAs("bob", assert(sql(s"show tables from $db").collect().length === 0))
-      doAs("i_am_invisible", assert(sql(s"show tables from $db").collect().length === 0))
-      doAs("i_am_invisible", assert(sql(s"show tables from $db").limit(1).isEmpty))
+      doAs(admin, assert(sql(s"show tables from $db").collect().length === 2))
+      doAs(bob, assert(sql(s"show tables from $db").collect().length === 0))
+      doAs(invisibleUser, assert(sql(s"show tables from $db").collect().length === 0))
+      doAs(invisibleUser, assert(sql(s"show tables from $db").limit(1).isEmpty))
     }
   }
 
@@ -318,19 +258,19 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     val db = "default2"
 
     withCleanTmpResources(Seq((db, "database"))) {
-      doAs("admin", sql(s"CREATE DATABASE IF NOT EXISTS $db"))
-      doAs("admin", assert(sql(s"SHOW DATABASES").collect().length == 2))
-      doAs("admin", assert(sql(s"SHOW DATABASES").collectAsList().get(0).getString(0) == "default"))
-      doAs("admin", assert(sql(s"SHOW DATABASES").collectAsList().get(1).getString(0) == s"$db"))
+      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $db"))
+      doAs(admin, assert(sql(s"SHOW DATABASES").collect().length == 2))
+      doAs(admin, assert(sql(s"SHOW DATABASES").collectAsList().get(0).getString(0) == defaultDb))
+      doAs(admin, assert(sql(s"SHOW DATABASES").collectAsList().get(1).getString(0) == s"$db"))
 
-      doAs("bob", assert(sql(s"SHOW DATABASES").collect().length == 1))
-      doAs("bob", assert(sql(s"SHOW DATABASES").collectAsList().get(0).getString(0) == "default"))
-      doAs("i_am_invisible", assert(sql(s"SHOW DATABASES").limit(1).isEmpty))
+      doAs(bob, assert(sql(s"SHOW DATABASES").collect().length == 1))
+      doAs(bob, assert(sql(s"SHOW DATABASES").collectAsList().get(0).getString(0) == defaultDb))
+      doAs(invisibleUser, assert(sql(s"SHOW DATABASES").limit(1).isEmpty))
     }
   }
 
   test("show functions") {
-    val default = "default"
+    val default = defaultDb
     val db3 = "default3"
     val function1 = "function1"
 
@@ -338,41 +278,41 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
       (s"$default.$function1", "function"),
       (s"$db3.$function1", "function"),
       (db3, "database"))) {
-      doAs("admin", sql(s"CREATE FUNCTION $function1 AS 'Function1'"))
-      doAs("admin", assert(sql(s"show user functions $default.$function1").collect().length == 1))
-      doAs("bob", assert(sql(s"show user functions $default.$function1").collect().length == 0))
+      doAs(admin, sql(s"CREATE FUNCTION $function1 AS 'Function1'"))
+      doAs(admin, assert(sql(s"show user functions $default.$function1").collect().length == 1))
+      doAs(bob, assert(sql(s"show user functions $default.$function1").collect().length == 0))
 
-      doAs("admin", sql(s"CREATE DATABASE IF NOT EXISTS $db3"))
-      doAs("admin", sql(s"CREATE FUNCTION $db3.$function1 AS 'Function1'"))
+      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $db3"))
+      doAs(admin, sql(s"CREATE FUNCTION $db3.$function1 AS 'Function1'"))
 
-      doAs("admin", assert(sql(s"show user functions $db3.$function1").collect().length == 1))
-      doAs("bob", assert(sql(s"show user functions $db3.$function1").collect().length == 0))
+      doAs(admin, assert(sql(s"show user functions $db3.$function1").collect().length == 1))
+      doAs(bob, assert(sql(s"show user functions $db3.$function1").collect().length == 0))
 
-      doAs("admin", assert(sql(s"show system functions").collect().length > 0))
-      doAs("bob", assert(sql(s"show system functions").collect().length > 0))
+      doAs(admin, assert(sql(s"show system functions").collect().length > 0))
+      doAs(bob, assert(sql(s"show system functions").collect().length > 0))
 
-      val adminSystemFunctionCount = doAs("admin", sql(s"show system functions").collect().length)
-      val bobSystemFunctionCount = doAs("bob", sql(s"show system functions").collect().length)
+      val adminSystemFunctionCount = doAs(admin, sql(s"show system functions").collect().length)
+      val bobSystemFunctionCount = doAs(bob, sql(s"show system functions").collect().length)
       assert(adminSystemFunctionCount == bobSystemFunctionCount)
     }
   }
 
   test("show columns") {
-    val db = "default"
+    val db = defaultDb
     val table = "src"
     val col = "key"
     val create = s"CREATE TABLE IF NOT EXISTS $db.$table ($col int, value int) USING $format"
 
     withCleanTmpResources(Seq((s"$db.$table", "table"))) {
-      doAs("admin", sql(create))
+      doAs(admin, sql(create))
 
-      doAs("admin", assert(sql(s"SHOW COLUMNS IN $table").count() == 2))
-      doAs("admin", assert(sql(s"SHOW COLUMNS IN $db.$table").count() == 2))
-      doAs("admin", assert(sql(s"SHOW COLUMNS IN $table IN $db").count() == 2))
+      doAs(admin, assert(sql(s"SHOW COLUMNS IN $table").count() == 2))
+      doAs(admin, assert(sql(s"SHOW COLUMNS IN $db.$table").count() == 2))
+      doAs(admin, assert(sql(s"SHOW COLUMNS IN $table IN $db").count() == 2))
 
-      doAs("kent", assert(sql(s"SHOW COLUMNS IN $table").count() == 1))
-      doAs("kent", assert(sql(s"SHOW COLUMNS IN $db.$table").count() == 1))
-      doAs("kent", assert(sql(s"SHOW COLUMNS IN $table IN $db").count() == 1))
+      doAs(kent, assert(sql(s"SHOW COLUMNS IN $table").count() == 1))
+      doAs(kent, assert(sql(s"SHOW COLUMNS IN $db.$table").count() == 1))
+      doAs(kent, assert(sql(s"SHOW COLUMNS IN $table IN $db").count() == 1))
     }
   }
 
@@ -387,24 +327,24 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
       (s"$db.${table}_select2", "table"),
       (s"$db.${table}_select3", "table"),
       (s"$db", "database"))) {
-      doAs("admin", sql(s"CREATE DATABASE IF NOT EXISTS $db"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_use1 (key int) USING $format"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_use2 (key int) USING $format"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select1 (key int) USING $format"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select2 (key int) USING $format"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select3 (key int) USING $format"))
+      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $db"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_use1 (key int) USING $format"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_use2 (key int) USING $format"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select1 (key int) USING $format"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select2 (key int) USING $format"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.${table}_select3 (key int) USING $format"))
 
       doAs(
-        "admin",
+        admin,
         assert(sql(s"show table extended from $db like '$table*'").collect().length === 5))
       doAs(
-        "bob",
+        bob,
         assert(sql(s"show tables from $db").collect().length === 5))
       doAs(
-        "bob",
+        bob,
         assert(sql(s"show table extended from $db like '$table*'").collect().length === 3))
       doAs(
-        "i_am_invisible",
+        invisibleUser,
         assert(sql(s"show table extended from $db like '$table*'").collect().length === 0))
     }
   }
@@ -416,48 +356,48 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     val globalTempView2 = "global_temp_view2"
 
     // create or replace view
-    doAs("denyuser", sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
+    doAs(denyUser, sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE GLOBAL TEMPORARY VIEW $globalTempView AS SELECT * FROM values(1)"))
 
     // rename view
-    doAs("denyuser2", sql(s"ALTER VIEW $tempView RENAME TO $tempView2"))
+    doAs(denyUser2, sql(s"ALTER VIEW $tempView RENAME TO $tempView2"))
     doAs(
-      "denyuser2",
+      denyUser2,
       sql(s"ALTER VIEW global_temp.$globalTempView RENAME TO global_temp.$globalTempView2"))
 
-    doAs("admin", sql(s"DROP VIEW IF EXISTS $tempView2"))
-    doAs("admin", sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView2"))
-    doAs("admin", assert(sql("show tables from global_temp").collect().length == 0))
+    doAs(admin, sql(s"DROP VIEW IF EXISTS $tempView2"))
+    doAs(admin, sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView2"))
+    doAs(admin, assert(sql("show tables from global_temp").collect().length == 0))
   }
 
   test("[KYUUBI #3426] Drop temp view should be skipped permission check") {
     val tempView = "temp_view"
     val globalTempView = "global_temp_view"
-    doAs("denyuser", sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
+    doAs(denyUser, sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
 
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE OR REPLACE TEMPORARY VIEW $tempView" +
         s" AS select * from values(1)"))
 
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE GLOBAL TEMPORARY VIEW $globalTempView AS SELECT * FROM values(1)"))
 
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE OR REPLACE GLOBAL TEMPORARY VIEW $globalTempView" +
         s" AS select * from values(1)"))
 
     // global_temp will contain the temporary view, even if it is not global
-    doAs("admin", assert(sql("show tables from global_temp").collect().length == 2))
+    doAs(admin, assert(sql("show tables from global_temp").collect().length == 2))
 
-    doAs("denyuser2", sql(s"DROP VIEW IF EXISTS $tempView"))
-    doAs("denyuser2", sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView"))
+    doAs(denyUser2, sql(s"DROP VIEW IF EXISTS $tempView"))
+    doAs(denyUser2, sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView"))
 
-    doAs("admin", assert(sql("show tables from global_temp").collect().length == 0))
+    doAs(admin, assert(sql("show tables from global_temp").collect().length == 0))
   }
 
   test("[KYUUBI #3428] AlterViewAsCommand should be skipped permission check") {
@@ -465,26 +405,26 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
     val globalTempView = "global_temp_view"
 
     // create or replace view
-    doAs("denyuser", sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
+    doAs(denyUser, sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)"))
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE OR REPLACE TEMPORARY VIEW $tempView" +
         s" AS select * from values(1)"))
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE GLOBAL TEMPORARY VIEW $globalTempView AS SELECT * FROM values(1)"))
     doAs(
-      "denyuser",
+      denyUser,
       sql(s"CREATE OR REPLACE GLOBAL TEMPORARY VIEW $globalTempView" +
         s" AS select * from values(1)"))
 
     // rename view
-    doAs("denyuser2", sql(s"ALTER VIEW $tempView AS SELECT * FROM values(1)"))
-    doAs("denyuser2", sql(s"ALTER VIEW global_temp.$globalTempView AS SELECT * FROM values(1)"))
+    doAs(denyUser2, sql(s"ALTER VIEW $tempView AS SELECT * FROM values(1)"))
+    doAs(denyUser2, sql(s"ALTER VIEW global_temp.$globalTempView AS SELECT * FROM values(1)"))
 
-    doAs("admin", sql(s"DROP VIEW IF EXISTS $tempView"))
-    doAs("admin", sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView"))
-    doAs("admin", assert(sql("show tables from global_temp").collect().length == 0))
+    doAs(admin, sql(s"DROP VIEW IF EXISTS $tempView"))
+    doAs(admin, sql(s"DROP VIEW IF EXISTS global_temp.$globalTempView"))
+    doAs(admin, assert(sql("show tables from global_temp").collect().length == 0))
   }
 
   test("[KYUUBI #3343] pass temporary view creation") {
@@ -493,28 +433,39 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
 
     withTempView(tempView) {
       doAs(
-        "denyuser",
+        denyUser,
         assert(Try(sql(s"CREATE TEMPORARY VIEW $tempView AS select * from values(1)")).isSuccess))
 
       doAs(
-        "denyuser",
+        denyUser,
         Try(sql(s"CREATE OR REPLACE TEMPORARY VIEW $tempView" +
           s" AS select * from values(1)")).isSuccess)
     }
 
     withGlobalTempView(globalTempView) {
       doAs(
-        "denyuser",
+        denyUser,
         Try(
           sql(
             s"CREATE GLOBAL TEMPORARY VIEW $globalTempView AS SELECT * FROM values(1)")).isSuccess)
 
       doAs(
-        "denyuser",
+        denyUser,
         Try(sql(s"CREATE OR REPLACE GLOBAL TEMPORARY VIEW $globalTempView" +
           s" AS select * from values(1)")).isSuccess)
     }
-    doAs("admin", assert(sql("show tables from global_temp").collect().length == 0))
+    doAs(admin, assert(sql("show tables from global_temp").collect().length == 0))
+  }
+
+  test("[KYUUBI #5172] Check USE permissions for DESCRIBE FUNCTION") {
+    val fun = s"$defaultDb.function1"
+
+    withCleanTmpResources(Seq((s"$fun", "function"))) {
+      doAs(admin, sql(s"CREATE FUNCTION $fun AS 'Function1'"))
+      doAs(admin, sql(s"DESC FUNCTION $fun").collect().length == 1)
+      val e = intercept[AccessControlException](doAs(denyUser, sql(s"DESC FUNCTION $fun")))
+      assert(e.getMessage === errorMessage("_any", "default/function1", denyUser))
+    }
   }
 }
 
@@ -527,12 +478,12 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   test("table stats must be specified") {
     val table = "hive_src"
     withCleanTmpResources(Seq((table, "table"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $table (id int)"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $table (id int)"))
       doAs(
-        "admin", {
+        admin, {
           val hiveTableRelation = sql(s"SELECT * FROM $table")
             .queryExecution.optimizedPlan.collectLeaves().head.asInstanceOf[HiveTableRelation]
-          assert(getFieldVal[Option[Statistics]](hiveTableRelation, "tableStats").nonEmpty)
+          assert(getField[Option[Statistics]](hiveTableRelation, "tableStats").nonEmpty)
         })
     }
   }
@@ -540,9 +491,9 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   test("HiveTableRelation should be able to be converted to LogicalRelation") {
     val table = "hive_src"
     withCleanTmpResources(Seq((table, "table"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $table (id int) STORED AS PARQUET"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $table (id int) STORED AS PARQUET"))
       doAs(
-        "admin", {
+        admin, {
           val relation = sql(s"SELECT * FROM $table")
             .queryExecution.optimizedPlan.collectLeaves().head
           assert(relation.isInstanceOf[LogicalRelation])
@@ -560,7 +511,7 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
       (s"$db.$table1", "table"),
       (s"$db", "database"))) {
       doAs(
-        "admin", {
+        admin, {
           sql(s"CREATE DATABASE IF NOT EXISTS $db")
           sql(s"CREATE TABLE IF NOT EXISTS $db.$table1(id int) STORED AS PARQUET")
           sql(s"INSERT INTO $db.$table1 SELECT 1")
@@ -581,16 +532,16 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
       (adminPermView, "view"),
       (permView, "view"),
       (table, "table"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $table (id int)"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $table (id int)"))
 
-      doAs("admin", sql(s"CREATE VIEW ${adminPermView} AS SELECT * FROM $table"))
+      doAs(admin, sql(s"CREATE VIEW ${adminPermView} AS SELECT * FROM $table"))
 
       val e1 = intercept[AccessControlException](
-        doAs("someone", sql(s"CREATE VIEW $permView AS SELECT 1 as a")))
+        doAs(someone, sql(s"CREATE VIEW $permView AS SELECT 1 as a")))
       assert(e1.getMessage.contains(s"does not have [create] privilege on [default/$permView]"))
 
       val e2 = intercept[AccessControlException](
-        doAs("someone", sql(s"CREATE VIEW $permView AS SELECT * FROM $table")))
+        doAs(someone, sql(s"CREATE VIEW $permView AS SELECT * FROM $table")))
       if (isSparkV32OrGreater) {
         assert(e2.getMessage.contains(s"does not have [select] privilege on [default/$table/id]"))
       } else {
@@ -600,20 +551,20 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("[KYUUBI #3326] check persisted view and skip shadowed table") {
-    val db1 = "default"
+    val db1 = defaultDb
     val table = "hive_src"
     val permView = "perm_view"
 
     withCleanTmpResources(Seq(
       (s"$db1.$table", "table"),
       (s"$db1.$permView", "view"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
-      doAs("admin", sql(s"CREATE VIEW $db1.$permView AS SELECT * FROM $db1.$table"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
+      doAs(admin, sql(s"CREATE VIEW $db1.$permView AS SELECT * FROM $db1.$table"))
 
       // KYUUBI #3326: with no privileges to the permanent view or the source table
       val e1 = intercept[AccessControlException](
         doAs(
-          "someone", {
+          someone, {
             sql(s"select * from $db1.$permView").collect()
           }))
       if (isSparkV31OrGreater) {
@@ -625,16 +576,16 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("KYUUBI #4504: query permanent view with privilege to permanent view only") {
-    val db1 = "default"
+    val db1 = defaultDb
     val table = "hive_src"
     val permView = "perm_view"
-    val userPermViewOnly = "user_perm_view_only"
+    val userPermViewOnly = permViewOnlyUser
 
     withCleanTmpResources(Seq(
       (s"$db1.$table", "table"),
       (s"$db1.$permView", "view"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
-      doAs("admin", sql(s"CREATE VIEW $db1.$permView AS SELECT * FROM $db1.$table"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
+      doAs(admin, sql(s"CREATE VIEW $db1.$permView AS SELECT * FROM $db1.$table"))
 
       // query all columns of the permanent view
       // with access privileges to the permanent view but no privilege to the source table
@@ -659,7 +610,7 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("[KYUUBI #3371] support throws all disallowed privileges in exception") {
-    val db1 = "default"
+    val db1 = defaultDb
     val srcTable1 = "hive_src1"
     val srcTable2 = "hive_src2"
     val sinkTable1 = "hive_sink1"
@@ -669,17 +620,17 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
       (s"$db1.$srcTable2", "table"),
       (s"$db1.$sinkTable1", "table"))) {
       doAs(
-        "admin",
+        admin,
         sql(s"CREATE TABLE IF NOT EXISTS $db1.$srcTable1" +
           s" (id int, name string, city string)"))
 
       doAs(
-        "admin",
+        admin,
         sql(s"CREATE TABLE IF NOT EXISTS $db1.$srcTable2" +
           s" (id int, age int)"))
 
       doAs(
-        "admin",
+        admin,
         sql(s"CREATE TABLE IF NOT EXISTS $db1.$sinkTable1" +
           s" (id int, age int, name string, city string)"))
 
@@ -688,25 +639,17 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
         s" FROM $db1.$srcTable1 as tb1" +
         s" JOIN $db1.$srcTable2 as tb2" +
         s" on tb1.id = tb2.id"
-      val e1 = intercept[AccessControlException](doAs("someone", sql(insertSql1)))
+      val e1 = intercept[AccessControlException](doAs(someone, sql(insertSql1)))
       assert(e1.getMessage.contains(s"does not have [select] privilege on [$db1/$srcTable1/id]"))
 
-      try {
-        SparkRangerAdminPlugin.getRangerConf.setBoolean(
-          s"ranger.plugin.${SparkRangerAdminPlugin.getServiceType}.authorize.in.single.call",
-          true)
-        val e2 = intercept[AccessControlException](doAs("someone", sql(insertSql1)))
+      withSingleCallEnabled {
+        val e2 = intercept[AccessControlException](doAs(someone, sql(insertSql1)))
         assert(e2.getMessage.contains(s"does not have" +
           s" [select] privilege on" +
           s" [$db1/$srcTable1/id,$db1/$srcTable1/name,$db1/$srcTable1/city," +
           s"$db1/$srcTable2/age,$db1/$srcTable2/id]," +
           s" [update] privilege on [$db1/$sinkTable1/id,$db1/$sinkTable1/age," +
           s"$db1/$sinkTable1/name,$db1/$sinkTable1/city]"))
-      } finally {
-        // revert to default value
-        SparkRangerAdminPlugin.getRangerConf.setBoolean(
-          s"ranger.plugin.${SparkRangerAdminPlugin.getServiceType}.authorize.in.single.call",
-          false)
       }
     }
   }
@@ -714,7 +657,7 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   test("[KYUUBI #3411] skip checking cache table") {
     if (isSparkV32OrGreater) { // cache table sql supported since 3.2.0
 
-      val db1 = "default"
+      val db1 = defaultDb
       val srcTable1 = "hive_src1"
       val cacheTable1 = "cacheTable1"
       val cacheTable2 = "cacheTable2"
@@ -729,23 +672,23 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
         (s"$db1.$cacheTable4", "cache"))) {
 
         doAs(
-          "admin",
+          admin,
           sql(s"CREATE TABLE IF NOT EXISTS $db1.$srcTable1" +
             s" (id int, name string, city string)"))
 
         val e1 = intercept[AccessControlException](
-          doAs("someone", sql(s"CACHE TABLE $cacheTable2 select * from $db1.$srcTable1")))
+          doAs(someone, sql(s"CACHE TABLE $cacheTable2 select * from $db1.$srcTable1")))
         assert(
           e1.getMessage.contains(s"does not have [select] privilege on [$db1/$srcTable1/id]"))
 
-        doAs("admin", sql(s"CACHE TABLE $cacheTable3 SELECT 1 AS a, 2 AS b "))
-        doAs("someone", sql(s"CACHE TABLE $cacheTable4 select 1 as a, 2 as b "))
+        doAs(admin, sql(s"CACHE TABLE $cacheTable3 SELECT 1 AS a, 2 AS b "))
+        doAs(someone, sql(s"CACHE TABLE $cacheTable4 select 1 as a, 2 as b "))
       }
     }
   }
 
   test("[KYUUBI #3608] Support {OWNER} variable for queries") {
-    val db = "default"
+    val db = defaultDb
     val table = "owner_variable"
 
     val select = s"SELECT key FROM $db.$table"
@@ -764,7 +707,7 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
         }.isSuccess))
 
       doAs(
-        "create_only_user", {
+        createOnlyUser, {
           val e = intercept[AccessControlException](sql(select).collect())
           assert(e.getMessage === errorMessage("select", s"$db/$table/key"))
         })
@@ -778,22 +721,22 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
       Seq(
         (s"$db.$table", "table"),
         (s"$db", "database"))) {
-      doAs("admin", sql(s"CREATE DATABASE IF NOT EXISTS $db"))
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db.$table (key int) USING $format"))
+      doAs(admin, sql(s"CREATE DATABASE IF NOT EXISTS $db"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db.$table (key int) USING $format"))
       sql("SHOW DATABASES").queryExecution.optimizedPlan.stats
       sql(s"SHOW TABLES IN $db").queryExecution.optimizedPlan.stats
     }
   }
 
   test("[KYUUBI #4658] insert overwrite hive directory") {
-    val db1 = "default"
+    val db1 = defaultDb
     val table = "src"
 
     withCleanTmpResources(Seq((s"$db1.$table", "table"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
       val e = intercept[AccessControlException](
         doAs(
-          "someone",
+          someone,
           sql(
             s"""INSERT OVERWRITE DIRECTORY '/tmp/test_dir' ROW FORMAT DELIMITED FIELDS
                | TERMINATED BY ','
@@ -803,14 +746,14 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("[KYUUBI #4658] insert overwrite datasource directory") {
-    val db1 = "default"
+    val db1 = defaultDb
     val table = "src"
 
     withCleanTmpResources(Seq((s"$db1.$table", "table"))) {
-      doAs("admin", sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
+      doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db1.$table (id int, name string)"))
       val e = intercept[AccessControlException](
         doAs(
-          "someone",
+          someone,
           sql(
             s"""INSERT OVERWRITE DIRECTORY '/tmp/test_dir'
                | USING parquet
