@@ -26,7 +26,6 @@ import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.{DataFrame, Dataset, Row, SparkSession}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.execution.{CollectLimitExec, LocalTableScanExec, SparkPlan, SQLExecution}
-import org.apache.spark.sql.execution.{CollectLimitExec, SparkPlan, SQLExecution}
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanExec
 import org.apache.spark.sql.execution.arrow.KyuubiArrowConverters
 import org.apache.spark.sql.execution.metric.{SQLMetric, SQLMetrics}
@@ -35,7 +34,9 @@ import org.apache.spark.sql.types._
 
 import org.apache.kyuubi.engine.spark.KyuubiSparkUtil
 import org.apache.kyuubi.engine.spark.schema.RowSet
-import org.apache.kyuubi.reflection.DynMethods
+import org.apache.kyuubi.engine.spark.util.SparkCatalogUtils.quoteIfNeeded
+import org.apache.kyuubi.util.reflect.DynMethods
+import org.apache.kyuubi.util.reflect.ReflectUtils._
 
 object SparkDatasetHelper extends Logging {
 
@@ -132,18 +133,6 @@ object SparkDatasetHelper extends Logging {
     df.select(cols: _*)
   }
 
-  /**
-   * Fork from Apache Spark-3.3.1 org.apache.spark.sql.catalyst.util.quoteIfNeeded to adapt to
-   * Spark-3.1.x
-   */
-  private def quoteIfNeeded(part: String): String = {
-    if (part.matches("[a-zA-Z0-9_]+") && !part.matches("\\d+")) {
-      part
-    } else {
-      s"`${part.replace("`", "``")}`"
-    }
-  }
-
   private lazy val maxBatchSize: Long = {
     // respect spark connect config
     KyuubiSparkUtil.globalSparkContext
@@ -236,15 +225,9 @@ object SparkDatasetHelper extends Logging {
   private def withFinalPlanUpdate[T](
       adaptiveSparkPlanExec: AdaptiveSparkPlanExec,
       fun: SparkPlan => T): T = {
-    val getFinalPhysicalPlan = DynMethods.builder("getFinalPhysicalPlan")
-      .hiddenImpl(adaptiveSparkPlanExec.getClass)
-      .build()
-    val plan = getFinalPhysicalPlan.invoke[SparkPlan](adaptiveSparkPlanExec)
+    val plan = invokeAs[SparkPlan](adaptiveSparkPlanExec, "getFinalPhysicalPlan")
     val result = fun(plan)
-    val finalPlanUpdate = DynMethods.builder("finalPlanUpdate")
-      .hiddenImpl(adaptiveSparkPlanExec.getClass)
-      .build()
-    finalPlanUpdate.invoke[Unit](adaptiveSparkPlanExec)
+    invokeAs[Unit](adaptiveSparkPlanExec, "finalPlanUpdate")
     result
   }
 
