@@ -19,7 +19,6 @@ package org.apache.kyuubi.kubernetes.test.spark
 
 import java.util.UUID
 
-import scala.collection.JavaConverters._
 import scala.concurrent.duration._
 
 import org.apache.hadoop.conf.Configuration
@@ -28,8 +27,8 @@ import org.apache.hadoop.net.NetUtils
 import org.apache.kyuubi._
 import org.apache.kyuubi.client.util.BatchUtils._
 import org.apache.kyuubi.config.KyuubiConf
-import org.apache.kyuubi.config.KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_HOST
-import org.apache.kyuubi.engine.{ApplicationInfo, ApplicationOperation, KubernetesApplicationOperation}
+import org.apache.kyuubi.config.KyuubiConf._
+import org.apache.kyuubi.engine.{ApplicationInfo, ApplicationManagerInfo, ApplicationOperation, KubernetesApplicationOperation}
 import org.apache.kyuubi.engine.ApplicationState.{FAILED, NOT_FOUND, RUNNING}
 import org.apache.kyuubi.engine.spark.SparkProcessBuilder
 import org.apache.kyuubi.kubernetes.test.MiniKube
@@ -44,20 +43,23 @@ abstract class SparkOnKubernetesSuiteBase
     MiniKube.getKubernetesClient.getMasterUrl.toString
   }
 
+  protected val appMgrInfo =
+    ApplicationManagerInfo(Some(s"k8s://$apiServerAddress"), Some("minikube"), None)
+
   protected def sparkOnK8sConf: KyuubiConf = {
     // TODO Support more Spark version
     // Spark official docker image: https://hub.docker.com/r/apache/spark/tags
     KyuubiConf().set("spark.master", s"k8s://$apiServerAddress")
-      .set("spark.kubernetes.container.image", "apache/spark:v3.3.2")
+      .set("spark.kubernetes.container.image", "apache/spark:3.4.1")
       .set("spark.kubernetes.container.image.pullPolicy", "IfNotPresent")
       .set("spark.executor.instances", "1")
       .set("spark.executor.memory", "512M")
       .set("spark.driver.memory", "512M")
       .set("spark.kubernetes.driver.request.cores", "250m")
       .set("spark.kubernetes.executor.request.cores", "250m")
-      .set("kyuubi.kubernetes.context", "minikube")
-      .set("kyuubi.frontend.protocols", "THRIFT_BINARY,REST")
-      .set("kyuubi.session.engine.initialize.timeout", "PT10M")
+      .set(KUBERNETES_CONTEXT.key, "minikube")
+      .set(FRONTEND_PROTOCOLS.key, "THRIFT_BINARY,REST")
+      .set(ENGINE_INIT_TIMEOUT.key, "PT10M")
   }
 }
 
@@ -146,24 +148,31 @@ class KyuubiOperationKubernetesClusterClientModeSuite
       "kyuubi",
       "passwd",
       "localhost",
-      batchRequest.getConf.asScala.toMap,
       batchRequest)
 
     eventually(timeout(3.minutes), interval(50.milliseconds)) {
-      val state = k8sOperation.getApplicationInfoByTag(sessionHandle.identifier.toString)
+      val state = k8sOperation.getApplicationInfoByTag(
+        appMgrInfo,
+        sessionHandle.identifier.toString)
       assert(state.id != null)
       assert(state.name != null)
       assert(state.state == RUNNING)
     }
 
-    val killResponse = k8sOperation.killApplicationByTag(sessionHandle.identifier.toString)
+    val killResponse = k8sOperation.killApplicationByTag(
+      appMgrInfo,
+      sessionHandle.identifier.toString)
     assert(killResponse._1)
     assert(killResponse._2 startsWith "Succeeded to terminate:")
 
-    val appInfo = k8sOperation.getApplicationInfoByTag(sessionHandle.identifier.toString)
+    val appInfo = k8sOperation.getApplicationInfoByTag(
+      appMgrInfo,
+      sessionHandle.identifier.toString)
     assert(appInfo == ApplicationInfo(null, null, NOT_FOUND))
 
-    val failKillResponse = k8sOperation.killApplicationByTag(sessionHandle.identifier.toString)
+    val failKillResponse = k8sOperation.killApplicationByTag(
+      appMgrInfo,
+      sessionHandle.identifier.toString)
     assert(!failKillResponse._1)
     assert(failKillResponse._2 === ApplicationOperation.NOT_FOUND)
   }
@@ -206,30 +215,37 @@ class KyuubiOperationKubernetesClusterClusterModeSuite
       "runner",
       "passwd",
       "localhost",
-      batchRequest.getConf.asScala.toMap,
       batchRequest)
 
     // wait for driver pod start
     eventually(timeout(3.minutes), interval(5.second)) {
       // trigger k8sOperation init here
-      val appInfo = k8sOperation.getApplicationInfoByTag(sessionHandle.identifier.toString)
+      val appInfo = k8sOperation.getApplicationInfoByTag(
+        appMgrInfo,
+        sessionHandle.identifier.toString)
       assert(appInfo.state == RUNNING)
       assert(appInfo.name.startsWith(driverPodNamePrefix))
     }
 
-    val killResponse = k8sOperation.killApplicationByTag(sessionHandle.identifier.toString)
+    val killResponse = k8sOperation.killApplicationByTag(
+      appMgrInfo,
+      sessionHandle.identifier.toString)
     assert(killResponse._1)
     assert(killResponse._2 endsWith "is completed")
     assert(killResponse._2 contains sessionHandle.identifier.toString)
 
     eventually(timeout(3.minutes), interval(50.milliseconds)) {
-      val appInfo = k8sOperation.getApplicationInfoByTag(sessionHandle.identifier.toString)
+      val appInfo = k8sOperation.getApplicationInfoByTag(
+        appMgrInfo,
+        sessionHandle.identifier.toString)
       // We may kill engine start but not ready
       // An EOF Error occurred when the driver was starting
       assert(appInfo.state == FAILED || appInfo.state == NOT_FOUND)
     }
 
-    val failKillResponse = k8sOperation.killApplicationByTag(sessionHandle.identifier.toString)
+    val failKillResponse = k8sOperation.killApplicationByTag(
+      appMgrInfo,
+      sessionHandle.identifier.toString)
     assert(!failKillResponse._1)
   }
 }
