@@ -21,8 +21,10 @@ import java.io._
 import java.net.{Inet4Address, InetAddress, NetworkInterface}
 import java.nio.charset.StandardCharsets
 import java.nio.file.{Files, Path, Paths, StandardCopyOption}
+import java.security.PrivilegedAction
 import java.text.SimpleDateFormat
 import java.util.{Date, Properties, TimeZone, UUID}
+import java.util.concurrent.TimeUnit
 import java.util.concurrent.atomic.AtomicLong
 import java.util.concurrent.locks.Lock
 
@@ -202,6 +204,14 @@ object Utils extends Logging {
 
   def currentUser: String = UserGroupInformation.getCurrentUser.getShortUserName
 
+  def doAs[T](
+      proxyUser: String,
+      realUser: UserGroupInformation = UserGroupInformation.getCurrentUser)(f: () => T): T = {
+    UserGroupInformation.createProxyUser(proxyUser, realUser).doAs(new PrivilegedAction[T] {
+      override def run(): T = f()
+    })
+  }
+
   private val shortVersionRegex = """^(\d+\.\d+\.\d+)(.*)?$""".r
 
   /**
@@ -222,6 +232,11 @@ object Utils extends Logging {
    */
   val isWindows: Boolean = SystemUtils.IS_OS_WINDOWS
 
+  /**
+   * Whether the underlying operating system is MacOS.
+   */
+  val isMac: Boolean = SystemUtils.IS_OS_MAC
+
   /**
    * Indicates whether Kyuubi is currently running unit tests.
    */
@@ -389,6 +404,21 @@ object Utils extends Logging {
 
   def isOnK8s: Boolean = Files.exists(Paths.get("/var/run/secrets/kubernetes.io"))
 
+  /**
+   * Return a nice string representation of the exception. It will call "printStackTrace" to
+   * recursively generate the stack trace including the exception and its causes.
+   */
+  def prettyPrint(e: Throwable): String = {
+    if (e == null) {
+      ""
+    } else {
+      // Use e.printStackTrace here because e.getStackTrace doesn't include the cause
+      val stringWriter = new StringWriter()
+      e.printStackTrace(new PrintWriter(stringWriter))
+      stringWriter.toString
+    }
+  }
+
   def withLockRequired[T](lock: Lock)(block: => T): T = {
     try {
       lock.lock()
@@ -397,4 +427,26 @@ object Utils extends Logging {
       lock.unlock()
     }
   }
+
+  /**
+   * Try killing the process gracefully first, then forcibly if process does not exit in
+   * graceful period.
+   *
+   * @param process the being killed process
+   * @param gracefulPeriod the graceful killing period, in milliseconds
+   * @return the exit code if process exit normally, None if the process finally was killed
+   *         forcibly
+   */
+  def terminateProcess(process: java.lang.Process, gracefulPeriod: Long): Option[Int] = {
+    process.destroy()
+    if (process.waitFor(gracefulPeriod, TimeUnit.MILLISECONDS)) {
+      Some(process.exitValue())
+    } else {
+      warn(s"Process does not exit after $gracefulPeriod ms, try to forcibly kill. " +
+        "Staging files generated by the process may be retained!")
+      process.destroyForcibly()
+      None
+    }
+  }
+
 }
