@@ -19,21 +19,21 @@ package org.apache.kyuubi.ctl.util
 import scala.collection.JavaConverters._
 import scala.collection.mutable.ListBuffer
 
-import org.apache.kyuubi.client.api.v1.dto.{Batch, Engine, GetBatchesResponse, SessionData}
+import org.apache.kyuubi.client.api.v1.dto.{Batch, Engine, GetBatchesResponse, ServerData, SessionData}
 import org.apache.kyuubi.ctl.util.DateTimeUtils._
 import org.apache.kyuubi.ha.client.ServiceNodeInfo
 
 private[ctl] object Render {
 
-  def renderServiceNodesInfo(title: String, serviceNodeInfo: Seq[ServiceNodeInfo]): String = {
+  def renderServiceNodesInfo(title: String, serviceNodeInfo: Iterable[ServiceNodeInfo]): String = {
     val header = Array("Namespace", "Host", "Port", "Version")
-    val rows = serviceNodeInfo.sortBy(_.nodeName).map { sn =>
+    val rows = serviceNodeInfo.toSeq.sortBy(_.nodeName).map { sn =>
       Array(sn.namespace, sn.host, sn.port.toString, sn.version.getOrElse(""))
     }.toArray
     Tabulator.format(title, header, rows)
   }
 
-  def renderEngineNodesInfo(engineNodesInfo: Seq[Engine]): String = {
+  def renderEngineNodesInfo(engineNodesInfo: Iterable[Engine]): String = {
     val title = s"Engine Node List (total ${engineNodesInfo.size})"
     val header = Array("Namespace", "Instance", "Attributes")
     val rows = engineNodesInfo.map { engine =>
@@ -45,7 +45,20 @@ private[ctl] object Render {
     Tabulator.format(title, header, rows)
   }
 
-  def renderSessionDataListInfo(sessions: Seq[SessionData]): String = {
+  def renderServerNodesInfo(serverNodesInfo: Iterable[ServerData]): String = {
+    val title = s"Server Node List (total ${serverNodesInfo.size})"
+    val header = Array("Namespace", "Instance", "Attributes", "Status")
+    val rows = serverNodesInfo.map { server =>
+      Array(
+        server.getNamespace,
+        server.getInstance,
+        server.getAttributes.asScala.map { case (k, v) => s"$k=$v" }.mkString("\n"),
+        server.getStatus)
+    }.toArray
+    Tabulator.format(title, header, rows)
+  }
+
+  def renderSessionDataListInfo(sessions: Iterable[SessionData]): String = {
     val title = s"Live Session List (total ${sessions.size})"
     val header = Array(
       "Identifier",
