@@ -124,32 +124,39 @@ class MetadataManager extends AbstractService("MetadataManager") {
     }
   }
 
-  def getBatch(batchId: String): Batch = {
-    Option(getBatchSessionMetadata(batchId)).map(buildBatch).orNull
+  def getBatch(batchId: String): Option[Batch] = {
+    getBatchSessionMetadata(batchId).map(buildBatch)
   }
 
-  def getBatchSessionMetadata(batchId: String): Metadata = {
-    Option(withMetadataRequestMetrics(_metadataStore.getMetadata(batchId, true))).filter(
-      _.sessionType == SessionType.BATCH).orNull
+  def getBatchSessionMetadata(batchId: String): Option[Metadata] = {
+    Option(withMetadataRequestMetrics(_metadataStore.getMetadata(batchId)))
+      .filter(_.sessionType == SessionType.BATCH)
   }
 
-  def getBatches(
+  def getBatches(filter: MetadataFilter, from: Int, size: Int): Seq[Batch] = {
+    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size)).map(
+      buildBatch)
+  }
+
+  def countBatch(
       batchType: String,
       batchUser: String,
       batchState: String,
-      createTime: Long,
-      endTime: Long,
-      from: Int,
-      size: Int): Seq[Batch] = {
+      kyuubiInstance: String): Int = {
     val filter = MetadataFilter(
       sessionType = SessionType.BATCH,
       engineType = batchType,
       username = batchUser,
       state = batchState,
-      createTime = createTime,
-      endTime = endTime)
-    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size, true)).map(
-      buildBatch)
+      kyuubiInstance = kyuubiInstance)
+    withMetadataRequestMetrics(_metadataStore.countMetadata(filter))
+  }
+
+  def pickBatchForSubmitting(kyuubiInstance: String): Option[Metadata] =
+    withMetadataRequestMetrics(_metadataStore.pickMetadata(kyuubiInstance))
+
+  def cancelUnscheduledBatch(batchId: String): Boolean = {
+    _metadataStore.transformMetadataState(batchId, "INITIALIZED", "CANCELED")
   }
 
   def getBatchesRecoveryMetadata(
@@ -161,7 +168,7 @@ class MetadataManager extends AbstractService("MetadataManager") {
       sessionType = SessionType.BATCH,
       state = state,
       kyuubiInstance = kyuubiInstance)
-    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size, false))
+    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size))
   }
 
   def getPeerInstanceClosedBatchesMetadata(
@@ -174,7 +181,7 @@ class MetadataManager extends AbstractService("MetadataManager") {
       state = state,
       kyuubiInstance = kyuubiInstance,
       peerInstanceClosed = true)
-    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size, true))
+    withMetadataRequestMetrics(_metadataStore.getMetadataList(filter, from, size))
   }
 
   def updateMetadata(metadata: Metadata, asyncRetryOnError: Boolean = true): Unit = {
