@@ -19,9 +19,8 @@ package org.apache.kyuubi.engine
 
 import java.lang.management.ManagementFactory
 import java.time.Duration
-import java.util.{ServiceLoader, UUID}
+import java.util.UUID
 
-import scala.collection.JavaConverters._
 import scala.sys.process._
 
 import org.scalatest.concurrent.PatienceConfiguration.Timeout
@@ -31,18 +30,18 @@ import org.apache.kyuubi.{KyuubiFunSuite, Utils}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiConf.SESSION_IDLE_TIMEOUT
 import org.apache.kyuubi.engine.spark.SparkProcessBuilder
+import org.apache.kyuubi.util.reflect.ReflectUtils._
 
 class JpsApplicationOperationSuite extends KyuubiFunSuite {
-  private val operations = ServiceLoader.load(classOf[ApplicationOperation])
-    .asScala.filter(_.getClass.isAssignableFrom(classOf[JpsApplicationOperation]))
-  private val jps = operations.head
+  private val jps = loadFromServiceLoader[ApplicationOperation]()
+    .find(_.getClass.isAssignableFrom(classOf[JpsApplicationOperation])).get
   jps.initialize(null)
 
   test("JpsApplicationOperation with jstat") {
-    assert(jps.isSupported(None))
-    assert(jps.isSupported(Some("local")))
-    assert(!jps.killApplicationByTag(null)._1)
-    assert(!jps.killApplicationByTag("have a space")._1)
+    assert(jps.isSupported(ApplicationManagerInfo(None)))
+    assert(jps.isSupported(ApplicationManagerInfo(Some("local"))))
+    assert(!jps.killApplicationByTag(ApplicationManagerInfo(None), null)._1)
+    assert(!jps.killApplicationByTag(ApplicationManagerInfo(None), "have a space")._1)
     val currentProcess = ManagementFactory.getRuntimeMXBean.getName
     val currentPid = currentProcess.splitAt(currentProcess.indexOf("@"))._1
 
@@ -53,16 +52,16 @@ class JpsApplicationOperationSuite extends KyuubiFunSuite {
     }.start()
 
     eventually(Timeout(10.seconds)) {
-      val desc1 = jps.getApplicationInfoByTag("sun.tools.jstat.Jstat")
+      val desc1 = jps.getApplicationInfoByTag(ApplicationManagerInfo(None), "sun.tools.jstat.Jstat")
       assert(desc1.id != null)
       assert(desc1.name != null)
       assert(desc1.state == ApplicationState.RUNNING)
     }
 
-    jps.killApplicationByTag("sun.tools.jstat.Jstat")
+    jps.killApplicationByTag(ApplicationManagerInfo(None), "sun.tools.jstat.Jstat")
 
     eventually(Timeout(10.seconds)) {
-      val desc2 = jps.getApplicationInfoByTag("sun.tools.jstat.Jstat")
+      val desc2 = jps.getApplicationInfoByTag(ApplicationManagerInfo(None), "sun.tools.jstat.Jstat")
       assert(desc2.id == null)
       assert(desc2.name == null)
       assert(desc2.state == ApplicationState.NOT_FOUND)
@@ -79,27 +78,25 @@ class JpsApplicationOperationSuite extends KyuubiFunSuite {
     val builder = new SparkProcessBuilder(user, conf)
     builder.start
 
-    assert(jps.isSupported(builder.clusterManager()))
+    assert(jps.isSupported(ApplicationManagerInfo(builder.clusterManager())))
     eventually(Timeout(10.seconds)) {
-      val desc1 = jps.getApplicationInfoByTag(id)
+      val desc1 = jps.getApplicationInfoByTag(ApplicationManagerInfo(None), id)
       assert(desc1.id != null)
       assert(desc1.name != null)
-      assert(!desc1.name.contains("org.apache.spark.launcher.Main"))
-      assert(desc1.name.contains("org.apache.spark.deploy.SparkSubmit"))
       assert(desc1.state == ApplicationState.RUNNING)
-      val response = jps.killApplicationByTag(id)
+      val response = jps.killApplicationByTag(ApplicationManagerInfo(None), id)
       assert(response._1, response._2)
       assert(response._2 startsWith "Succeeded to terminate:")
     }
 
     eventually(Timeout(10.seconds)) {
-      val desc2 = jps.getApplicationInfoByTag(id)
+      val desc2 = jps.getApplicationInfoByTag(ApplicationManagerInfo(None), id)
       assert(desc2.id == null)
       assert(desc2.name == null)
       assert(desc2.state == ApplicationState.NOT_FOUND)
     }
 
-    val response2 = jps.killApplicationByTag(id)
+    val response2 = jps.killApplicationByTag(ApplicationManagerInfo(None), id)
     assert(!response2._1)
     assert(response2._2 === ApplicationOperation.NOT_FOUND)
   }
