@@ -17,32 +17,44 @@
 
 package org.apache.kyuubi.server.api.v1
 
-import java.nio.charset.StandardCharsets
-import java.util.{Base64, UUID}
+import java.time.Duration
+import java.util.UUID
+import javax.ws.rs.client.Entity
 import javax.ws.rs.core.{GenericType, MediaType}
 
+import scala.collection.JavaConverters._
+
+import org.apache.hive.service.rpc.thrift.TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V2
+import org.mockito.Mockito.lenient
 import org.scalatest.time.SpanSugar.convertIntToGrainOfTime
+import org.scalatestplus.mockito.MockitoSugar.mock
 
 import org.apache.kyuubi.{KYUUBI_VERSION, KyuubiFunSuite, RestFrontendTestHelper, Utils}
-import org.apache.kyuubi.client.api.v1.dto.Engine
+import org.apache.kyuubi.client.api.v1.dto._
 import org.apache.kyuubi.config.KyuubiConf
-import org.apache.kyuubi.engine.{ApplicationState, EngineRef, KyuubiApplicationManager}
+import org.apache.kyuubi.config.KyuubiConf._
+import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_SESSION_CONNECTION_URL_KEY
+import org.apache.kyuubi.engine.{ApplicationManagerInfo, ApplicationState, EngineRef, KyuubiApplicationManager}
 import org.apache.kyuubi.engine.EngineType.SPARK_SQL
 import org.apache.kyuubi.engine.ShareLevel.{CONNECTION, GROUP, USER}
 import org.apache.kyuubi.ha.HighAvailabilityConf
+import org.apache.kyuubi.ha.client.{DiscoveryPaths, ServiceDiscovery}
 import org.apache.kyuubi.ha.client.DiscoveryClientProvider.withDiscoveryClient
-import org.apache.kyuubi.ha.client.DiscoveryPaths
 import org.apache.kyuubi.plugin.PluginLoader
 import org.apache.kyuubi.server.KyuubiRestFrontendService
-import org.apache.kyuubi.server.http.authentication.AuthenticationHandler.AUTHORIZATION_HEADER
+import org.apache.kyuubi.server.http.util.HttpAuthUtils
+import org.apache.kyuubi.server.http.util.HttpAuthUtils.AUTHORIZATION_HEADER
+import org.apache.kyuubi.service.authentication.AnonymousAuthenticationProviderImpl
 
 class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
 
   private val engineMgr = new KyuubiApplicationManager()
 
-  val encodeAuthorization = new String(
-    Base64.getEncoder.encode(s"${Utils.currentUser}:".getBytes()),
-    StandardCharsets.UTF_8)
+  override protected lazy val conf: KyuubiConf = KyuubiConf()
+    .set(AUTHENTICATION_METHOD, Set("CUSTOM"))
+    .set(AUTHENTICATION_CUSTOM_CLASS, classOf[AnonymousAuthenticationProviderImpl].getName)
+    .set(SERVER_ADMINISTRATORS, Set("admin001"))
+    .set(ENGINE_IDLE_TIMEOUT, Duration.ofMinutes(3).toMillis)
 
   override def beforeAll(): Unit = {
     super.beforeAll()
@@ -59,39 +71,197 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     var response = webTarget.path("api/v1/admin/refresh/hadoop_conf")
       .request()
       .post(null)
-    assert(405 == response.getStatus)
+    assert(response.getStatus === 401)
 
     response = webTarget.path("api/v1/admin/refresh/hadoop_conf")
       .request()
-      .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
       .post(null)
-    assert(200 == response.getStatus)
+    assert(response.getStatus === 200)
+
+    response = webTarget.path("api/v1/admin/refresh/hadoop_conf")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader("admin001"))
+      .post(null)
+    assert(response.getStatus === 200)
+
+    response = webTarget.path("api/v1/admin/refresh/hadoop_conf")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader("admin002"))
+      .post(null)
+    assert(response.getStatus === 405)
   }
 
   test("refresh user defaults config of the kyuubi server") {
     var response = webTarget.path("api/v1/admin/refresh/user_defaults_conf")
       .request()
       .post(null)
-    assert(405 == response.getStatus)
+    assert(response.getStatus === 401)
 
     response = webTarget.path("api/v1/admin/refresh/user_defaults_conf")
       .request()
-      .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
       .post(null)
-    assert(200 == response.getStatus)
+    assert(response.getStatus === 200)
   }
 
   test("refresh unlimited users of the kyuubi server") {
     var response = webTarget.path("api/v1/admin/refresh/unlimited_users")
       .request()
       .post(null)
-    assert(405 == response.getStatus)
+    assert(response.getStatus === 401)
 
     response = webTarget.path("api/v1/admin/refresh/unlimited_users")
       .request()
-      .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
       .post(null)
-    assert(200 == response.getStatus)
+    assert(response.getStatus === 200)
+  }
+
+  test("refresh deny users of the kyuubi server") {
+    var response = webTarget.path("api/v1/admin/refresh/deny_users")
+      .request()
+      .post(null)
+    assert(response.getStatus === 401)
+
+    response = webTarget.path("api/v1/admin/refresh/deny_users")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .post(null)
+    assert(response.getStatus === 200)
+  }
+
+  test("list/close sessions") {
+    val requestObj = new SessionOpenRequest(Map("testConfig" -> "testValue").asJava)
+
+    var response = webTarget.path("api/v1/sessions")
+      .request(MediaType.APPLICATION_JSON_TYPE)
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .post(Entity.entity(requestObj, MediaType.APPLICATION_JSON_TYPE))
+    assert(response.getStatus === 200)
+
+    // get session list
+    var response2 = webTarget.path("api/v1/admin/sessions").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    assert(response2.getStatus === 200)
+    val sessions1 = response2.readEntity(new GenericType[Seq[SessionData]]() {})
+    assert(sessions1.nonEmpty)
+    assert(sessions1.head.getConf.get(KYUUBI_SESSION_CONNECTION_URL_KEY) === fe.connectionUrl)
+
+    // close an opened session
+    val sessionHandle = response.readEntity(classOf[SessionHandle]).getIdentifier
+    response = webTarget.path(s"api/v1/admin/sessions/$sessionHandle").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .delete()
+    assert(response.getStatus === 200)
+
+    // get session list again
+    response2 = webTarget.path("api/v1/admin/sessions").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    assert(200 == response2.getStatus)
+    val sessions2 = response2.readEntity(classOf[Seq[SessionData]])
+    assert(sessions2.isEmpty)
+  }
+
+  test("list sessions/operations with filter") {
+    fe.be.openSession(
+      HIVE_CLI_SERVICE_PROTOCOL_V2,
+      "admin",
+      "123456",
+      "localhost",
+      Map("testConfig" -> "testValue"))
+
+    fe.be.openSession(
+      HIVE_CLI_SERVICE_PROTOCOL_V2,
+      "admin",
+      "123456",
+      "localhost",
+      Map("testConfig" -> "testValue"))
+
+    fe.be.openSession(
+      HIVE_CLI_SERVICE_PROTOCOL_V2,
+      "test_user_1",
+      "xxxxxx",
+      "localhost",
+      Map("testConfig" -> "testValue"))
+
+    val sessionHandle = fe.be.openSession(
+      HIVE_CLI_SERVICE_PROTOCOL_V2,
+      "test_user_2",
+      "xxxxxx",
+      "localhost",
+      Map("testConfig" -> "testValue"))
+
+    // list sessions
+    var response = webTarget.path("api/v1/admin/sessions")
+      .queryParam("users", "admin")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    var sessions = response.readEntity(classOf[Seq[SessionData]])
+    assert(response.getStatus === 200)
+    assert(sessions.size == 2)
+
+    response = webTarget.path("api/v1/admin/sessions")
+      .queryParam("users", "test_user_1,test_user_2")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    sessions = response.readEntity(classOf[Seq[SessionData]])
+    assert(response.getStatus === 200)
+    assert(sessions.size == 2)
+
+    // list operations
+    response = webTarget.path("api/v1/admin/operations")
+      .queryParam("users", "test_user_1,test_user_2")
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    var operations = response.readEntity(classOf[Seq[OperationData]])
+    assert(operations.size == 2)
+
+    response = webTarget.path("api/v1/admin/operations")
+      .queryParam("sessionHandle", sessionHandle.identifier)
+      .request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    operations = response.readEntity(classOf[Seq[OperationData]])
+    assert(response.getStatus === 200)
+    assert(operations.size == 1)
+  }
+
+  test("list/close operations") {
+    val sessionHandle = fe.be.openSession(
+      HIVE_CLI_SERVICE_PROTOCOL_V2,
+      "admin",
+      "123456",
+      "localhost",
+      Map("testConfig" -> "testValue"))
+    val operation = fe.be.getCatalogs(sessionHandle)
+
+    // list operations
+    var response = webTarget.path("api/v1/admin/operations").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    assert(response.getStatus === 200)
+    var operations = response.readEntity(new GenericType[Seq[OperationData]]() {})
+    assert(operations.nonEmpty)
+    assert(operations.map(op => op.getIdentifier).contains(operation.identifier.toString))
+
+    // close operation
+    response = webTarget.path(s"api/v1/admin/operations/${operation.identifier}").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .delete()
+    assert(response.getStatus === 200)
+
+    // list again
+    response = webTarget.path("api/v1/admin/operations").request()
+      .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+      .get()
+    operations = response.readEntity(new GenericType[Seq[OperationData]]() {})
+    assert(!operations.map(op => op.getIdentifier).contains(operation.identifier.toString))
   }
 
   test("delete engine - user share level") {
@@ -100,7 +270,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val engine =
@@ -121,19 +290,20 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
         .queryParam("sharelevel", "USER")
         .queryParam("type", "spark_sql")
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .delete()
 
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
       assert(client.pathExists(engineSpace))
       eventually(timeout(5.seconds), interval(100.milliseconds)) {
-        assert(client.getChildren(engineSpace).size == 0, s"refId same with $id?")
+        assert(client.getChildren(engineSpace).isEmpty, s"refId same with $id?")
       }
 
       // kill the engine application
-      engineMgr.killApplication(None, id)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id)
       eventually(timeout(30.seconds), interval(100.milliseconds)) {
-        assert(engineMgr.getApplicationInfo(None, id).exists(_.state == ApplicationState.NOT_FOUND))
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id).exists(
+          _.state == ApplicationState.NOT_FOUND))
       }
     }
   }
@@ -144,7 +314,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val engine =
@@ -167,19 +336,20 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
         .queryParam("sharelevel", "GROUP")
         .queryParam("type", "spark_sql")
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .delete()
 
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
       assert(client.pathExists(engineSpace))
       eventually(timeout(5.seconds), interval(100.milliseconds)) {
-        assert(client.getChildren(engineSpace).size == 0, s"refId same with $id?")
+        assert(client.getChildren(engineSpace).isEmpty, s"refId same with $id?")
       }
 
       // kill the engine application
-      engineMgr.killApplication(None, id)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id)
       eventually(timeout(30.seconds), interval(100.milliseconds)) {
-        assert(engineMgr.getApplicationInfo(None, id).exists(_.state == ApplicationState.NOT_FOUND))
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id).exists(
+          _.state == ApplicationState.NOT_FOUND))
       }
     }
   }
@@ -189,7 +359,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val id = UUID.randomUUID().toString
@@ -211,10 +380,10 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
         .queryParam("type", "spark_sql")
         .queryParam("subdomain", id)
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .delete()
 
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
     }
   }
 
@@ -224,7 +393,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val engine =
@@ -244,10 +412,10 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       val response = webTarget.path("api/v1/admin/engine")
         .queryParam("type", "spark_sql")
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .get
 
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
       val engines = response.readEntity(new GenericType[Seq[Engine]]() {})
       assert(engines.size == 1)
       assert(engines(0).getEngineType == "SPARK_SQL")
@@ -255,9 +423,10 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       assert(engines(0).getSubdomain == "default")
 
       // kill the engine application
-      engineMgr.killApplication(None, id)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id)
       eventually(timeout(30.seconds), interval(100.milliseconds)) {
-        assert(engineMgr.getApplicationInfo(None, id).exists(_.state == ApplicationState.NOT_FOUND))
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id).exists(
+          _.state == ApplicationState.NOT_FOUND))
       }
     }
   }
@@ -268,7 +437,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val engine =
@@ -290,10 +458,10 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       val response = webTarget.path("api/v1/admin/engine")
         .queryParam("type", "spark_sql")
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .get
 
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
       val engines = response.readEntity(new GenericType[Seq[Engine]]() {})
       assert(engines.size == 1)
       assert(engines(0).getEngineType == "SPARK_SQL")
@@ -301,9 +469,10 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       assert(engines(0).getSubdomain == "default")
 
       // kill the engine application
-      engineMgr.killApplication(None, id)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id)
       eventually(timeout(30.seconds), interval(100.milliseconds)) {
-        assert(engineMgr.getApplicationInfo(None, id).exists(_.state == ApplicationState.NOT_FOUND))
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id).exists(
+          _.state == ApplicationState.NOT_FOUND))
       }
     }
   }
@@ -313,7 +482,6 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     conf.set(KyuubiConf.ENGINE_TYPE, SPARK_SQL.toString)
     conf.set(KyuubiConf.FRONTEND_THRIFT_BINARY_BIND_PORT, 0)
     conf.set(HighAvailabilityConf.HA_NAMESPACE, "kyuubi_test")
-    conf.set(KyuubiConf.ENGINE_IDLE_TIMEOUT, 180000L)
     conf.set(KyuubiConf.GROUP_PROVIDER, "hadoop")
 
     val engineSpace = DiscoveryPaths.makePath(
@@ -349,9 +517,9 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       val response = webTarget.path("api/v1/admin/engine")
         .queryParam("type", "spark_sql")
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .get
-      assert(200 == response.getStatus)
+      assert(response.getStatus === 200)
       val result = response.readEntity(new GenericType[Seq[Engine]]() {})
       assert(result.size == 2)
 
@@ -359,22 +527,53 @@ class AdminResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
         .queryParam("type", "spark_sql")
         .queryParam("subdomain", id1)
         .request(MediaType.APPLICATION_JSON_TYPE)
-        .header(AUTHORIZATION_HEADER, s"BASIC $encodeAuthorization")
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
         .get
       assert(200 == response1.getStatus)
       val result1 = response1.readEntity(new GenericType[Seq[Engine]]() {})
       assert(result1.size == 1)
 
       // kill the engine application
-      engineMgr.killApplication(None, id1)
-      engineMgr.killApplication(None, id2)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id1)
+      engineMgr.killApplication(ApplicationManagerInfo(None), id2)
       eventually(timeout(30.seconds), interval(100.milliseconds)) {
-        assert(engineMgr.getApplicationInfo(None, id1)
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id1)
           .exists(_.state == ApplicationState.NOT_FOUND))
-        assert(engineMgr.getApplicationInfo(None, id2)
+        assert(engineMgr.getApplicationInfo(ApplicationManagerInfo(None), id2)
           .exists(_.state == ApplicationState.NOT_FOUND))
       }
     }
   }
 
+  test("list server") {
+    // Mock Kyuubi Server
+    val serverDiscovery = mock[ServiceDiscovery]
+    lenient.when(serverDiscovery.fe).thenReturn(fe)
+    val namespace = conf.get(HighAvailabilityConf.HA_NAMESPACE)
+    withDiscoveryClient(conf) { client =>
+      client.registerService(conf, namespace, serverDiscovery)
+
+      val response = webTarget.path("api/v1/admin/server")
+        .request()
+        .header(AUTHORIZATION_HEADER, HttpAuthUtils.basicAuthorizationHeader(Utils.currentUser))
+        .get
+
+      assert(response.getStatus === 200)
+      val result = response.readEntity(new GenericType[Seq[ServerData]]() {})
+      assert(result.size == 1)
+      val testServer = result.head
+      val restFrontendService = fe.asInstanceOf[KyuubiRestFrontendService]
+
+      assert(namespace.equals(testServer.getNamespace.replaceFirst("/", "")))
+      assert(restFrontendService.host.equals(testServer.getHost))
+      assert(restFrontendService.connectionUrl.equals(testServer.getInstance()))
+      assert(!testServer.getAttributes.isEmpty)
+      val attributes = testServer.getAttributes
+      assert(attributes.containsKey("serviceUri") &&
+        attributes.get("serviceUri").equals(fe.connectionUrl))
+      assert(attributes.containsKey("version"))
+      assert(attributes.containsKey("sequence"))
+      assert("Running".equals(testServer.getStatus))
+    }
+  }
 }
